/* eslint-disable import/first, prefer-arrow-callback */
jest.mock('../AnueSearchFetch');
jest.mock('../../AnueSearchInput/AnueSearchInput', () => jest.fn(({ ...params }) => <input {...params} />));
jest.mock('../helpers');
jest.mock('../../../utils/anueCookie');
jest.mock('../../AnueSearchRealtimeResult/AnueSearchRealtimeResult');
jest.mock('../../../utils/debounce');
jest.mock('../../../utils/anueLink');

import React from 'react';
import { mount } from 'enzyme';
import AnueSearch from '../AnueSearch';
import AnueSearchFetch from '../AnueSearchFetch';
import AnueSearchInput from '../../AnueSearchInput/AnueSearchInput';
import AnueSearchRealtimeResult from '../../AnueSearchRealtimeResult/AnueSearchRealtimeResult';
import SearchResultHelper from '../helpers';
import AnueCookie from '../../../utils/anueCookie';
import debounce from '../../../utils/debounce';
import AnueLink from '../../../utils/anueLink';

describe('<AnueSearch /> component', () => {
  let makeSubject;
  const defaultProps = {
    maxRecentHistory: 5,
    theme: null,
    customAttribute: null,
    customWrapperStyles: null,
    customInputStyles: null,
    customIconStyles: null,
    shouldAlwaysDisplayInput: false,
    placeholder: '搜尋新聞、代碼或名稱',
    onIconClick: jest.fn(),
    onInputChange: jest.fn(),
    onInputFocus: jest.fn(),
    onInputBlur: jest.fn(),
    onInputPressEnter: jest.fn(),
  };

  beforeEach(() => {
    jest.resetModules();
    jest.resetAllMocks();
    AnueSearchInput.mockImplementation(({ onInputChange, onInputFocus, onInputBlur }) => (
      <input onChange={onInputChange} onFocus={onInputFocus} onBlur={onInputBlur} />
    ));
    AnueLink.mockImplementation(() => ({
      generateSearchResultLink: jest.fn(),
    }));
    AnueSearchRealtimeResult.mockImplementation(() => <div />);
    AnueCookie.mockImplementation(function _cookie() {
      return {
        setSearchCookies: jest.fn(),
        getSearchCookies: jest.fn(),
      };
    });
    SearchResultHelper.mockImplementation(function _helpers() {
      return {
        hotTagsResponseHelper: jest.fn(),
        hotTradesResponseHelper: jest.fn(),
        hotDriversResponseHelper: jest.fn(),
        recentSearchDataHelper: jest.fn(),
        searchTradesResponseHelper: jest.fn(),
      };
    });
    debounce.mockImplementation(fn => fn);
    makeSubject = params => mount(<AnueSearch {...defaultProps} {...params} />);
  });

  it('should fetch the hot search and retrieve cookies if the search input is focusing', async () => {
    const mockGet = jest.fn();
    const mockHotTagsResponseHelper = jest.fn();
    const mockHotTradesResponseHelper = jest.fn();
    const mockHotDriversResponseHelper = jest.fn();
    const mockRecentSearchDataHelper = jest.fn();
    const mockGetSearchCookies = jest.fn();

    AnueSearchFetch.mockImplementation(function _fetch() {
      return {
        get: mockGet,
      };
    });
    SearchResultHelper.mockImplementation(function _helpers() {
      return {
        hotTagsResponseHelper: mockHotTagsResponseHelper,
        hotTradesResponseHelper: mockHotTradesResponseHelper,
        hotDriversResponseHelper: mockHotDriversResponseHelper,
        recentSearchDataHelper: mockRecentSearchDataHelper,
      };
    });
    AnueCookie.mockImplementation(function _cookie() {
      return {
        getSearchCookies: mockGetSearchCookies,
      };
    });

    const subject = makeSubject();

    // Fetch
    expect(mockGet).not.toHaveBeenCalled();
    // State
    expect(subject.state().hover).toBe(false);
    expect(mockHotTagsResponseHelper).not.toHaveBeenCalled();
    expect(mockHotDriversResponseHelper).not.toHaveBeenCalled();
    expect(mockHotTradesResponseHelper).not.toHaveBeenCalled();
    expect(mockRecentSearchDataHelper).not.toHaveBeenCalled();
    expect(mockGetSearchCookies).not.toHaveBeenCalled();
    // Props
    expect(defaultProps.onInputFocus).not.toHaveBeenCalled();
    // document.body.style = null
    expect(document.body.style.length).toBe(0);

    subject.find('input').simulate('focus');

    // Fetch
    await expect(mockGet).toHaveBeenCalled();
    await expect(mockGet).toHaveBeenCalled();
    await expect(mockGet).toHaveBeenCalled();
    // State
    expect(subject.state().hover).toBe(true);
    expect(mockHotTagsResponseHelper).toHaveBeenCalled();
    expect(mockHotDriversResponseHelper).toHaveBeenCalled();
    expect(mockHotTradesResponseHelper).toHaveBeenCalled();
    expect(mockRecentSearchDataHelper).toHaveBeenCalled();
    expect(mockGetSearchCookies).toHaveBeenCalled();
    // Props
    expect(defaultProps.onInputFocus).toHaveBeenCalled();
    // document.body.style += overflow-y: hidden
    expect(document.body.style.length).toBe(1);
  });

  it('should call search api and store the search string to the coolie if the value of search input is changing', async () => {
    const mockGet = jest.fn();
    const mockSearchTradesResponseHelper = jest.fn(() => ({
      quoteFunds: [],
      news: [],
      oldDrivers: [],
    }));
    const mockSetSearchCookies = jest.fn();
    const mockGetSearchCookies = jest.fn();

    AnueSearchFetch.mockImplementation(function _fetch() {
      return {
        get: mockGet,
      };
    });
    SearchResultHelper.mockImplementation(function _helpers() {
      return {
        searchTradesResponseHelper: mockSearchTradesResponseHelper,
        recentSearchDataHelper: jest.fn(),
      };
    });
    AnueCookie.mockImplementation(function _cookie() {
      return {
        setSearchCookies: mockSetSearchCookies,
        getSearchCookies: mockGetSearchCookies,
      };
    });
    const subject = makeSubject();

    // Prop
    expect(defaultProps.onInputChange).not.toHaveBeenCalled();
    // Fetch
    expect(mockGet).not.toHaveBeenCalled();
    // State
    expect(mockSearchTradesResponseHelper).not.toHaveBeenCalled();
    expect(mockSetSearchCookies).not.toHaveBeenCalled();
    expect(mockGetSearchCookies).not.toHaveBeenCalled();

    subject.find('input').simulate('change', () => ({ target: { value: 123 } }));

    // Prop
    expect(defaultProps.onInputChange).toHaveBeenCalled();
    // Fetch
    await expect(mockGet).toHaveBeenCalled();
    // State
    expect(mockSearchTradesResponseHelper).toHaveBeenCalled();
  });

  it('should set the state.hover to false if click anywhere outside of the component', () => {
    const subject = makeSubject();

    expect(subject.state().hover).toBe(false);
    subject.find('input').simulate('focus');
    expect(subject.state().hover).toBe(true);

    document.dispatchEvent(new Event('click', { bubbles: true }));
    expect(subject.state().hover).toBe(false);
  });

  it('should go to search result page if click the search icon when the input has value', () => {
    const mockSetSearchCookies = jest.fn();
    const mockGetSearchCookies = jest.fn();
    const targetMockLink = 'https://www.cnyes.com/search';
    const mockGenerateSearchResultLink = jest.fn(() => targetMockLink);
    const originLocation = window.location;

    delete global.window.location;

    global.window.location = { href: 'Test 123' };
    AnueLink.mockImplementation(() => ({
      generateSearchResultLink: mockGenerateSearchResultLink,
    }));
    AnueCookie.mockImplementation(function _cookie() {
      return {
        setSearchCookies: mockSetSearchCookies,
        getSearchCookies: mockGetSearchCookies,
      };
    });
    AnueSearchInput.mockImplementation(({ onInputChange, onInputFocus, onInputBlur, onIconClick }) => (
      <input onClick={onIconClick} onChange={onInputChange} onFocus={onInputFocus} onBlur={onInputBlur} />
    ));
    const subject = makeSubject();

    subject.find('input').simulate('click');

    expect(defaultProps.onIconClick).toHaveBeenCalled();
    expect(mockSetSearchCookies).not.toHaveBeenCalled();
    expect(mockGetSearchCookies).not.toHaveBeenCalled();
    expect(window.location.href).not.toBe(targetMockLink);

    subject.setState({
      searchString: '123',
    });
    subject.find('input').simulate('click');

    expect(mockSetSearchCookies).toHaveBeenCalled();
    expect(mockGetSearchCookies).toHaveBeenCalled();
    expect(mockGenerateSearchResultLink).toHaveBeenCalled();
    expect(window.location.href).toBe(targetMockLink);

    global.window.location = originLocation;
  });
});
